// Copyright (C) 2008 David Bremner
//
// Author: David Bremner (bremner@unb.ca)
// Version: $Id$
//
// This file is part of lpmargin.
//
// lpmargin is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 2 of the License, or
// (at your option) any later version.
//
// lpmargin is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with lpmargin.  If not, see <http://www.gnu.org/licenses/>.
#ifndef _DB_VECTOR_H
#define _DB_VECTOR_H
#include "debug.h"

#include <iostream>
#include <iomanip>
#include <vector>

template <class T>
class Vector {
  typedef Vector<T> vec;

  std::vector<T> data;
  
 public: 
  typedef typename std::vector<T> stdvec;
  typedef T value;
  typedef typename stdvec::const_iterator const_iterator;
  typedef typename stdvec::iterator iterator;

  // Constructors

  Vector <T> () {};
  explicit Vector <T> (int size) : data(size) {  };
  Vector <T> (int size, int val) : data(size,static_cast<T>(val)) {  };
  
  // iterators
  iterator begin() { return data.begin(); };
  iterator end() { return data.end(); };
  const_iterator begin() const { return data.begin(); };
  const_iterator end() const { return data.end(); };

  // access
  value &operator [] (int i) { return data[i]; };
  const value &operator [] (int i) const { return data[i]; };

  // modify vectors
  void clear() {data.clear();};

  void append(const value & x){ data.push_back(x); };
  void append(const vec& v) { 
    typename stdvec::const_iterator j; 
    for (j=v.begin(); j!=v.end(); j++)
      append(*j);
  };

  size_t size() const { return data.size(); };
  // syntactic sugar

  void append(int i){append(static_cast<const T&>(i));};

  vec join(const vec& v) { vec w(*this);  w.append(v); return (w);};




  // (destructive) arithmetic.

  void mul (T scalar){ 
    typename stdvec::iterator j;
    for (j=data.begin();j!=data.end();j++){
      (*j) *= scalar;
    }
  }
  void add (const vec& v){ 
    for (int i=0; i<data.size(); i++){
      (*this)[i] += v[i];
    }
  }

  void sub (const vec& v){ 
    for (unsigned i=0; i<data.size(); i++){
      (*this)[i] -= v[i];
    }
  }

  // copying versions
  vec operator +(const vec& v) const{
    vec w(*this);
    w.add(v);
    return w;
  }

  vec operator -(const vec& v) const{
    vec w(*this);
    w.sub(v);
    return w;
  }


  // wrappers for integers
  vec operator *(int scalar){ return static_cast<T>(scalar) * (*this); };

};

template <class T>  Vector<T> operator * (const T& scalar, 
						  Vector<T> v){
  Vector<T> w=v;
  w.mul(scalar);
  return w;
}


template <class T>  Vector<T> operator * (int scalar, 
						  Vector<T> v){
  return static_cast<T>(scalar) * v;
}



using std::ostream;

template <typename T>
inline ostream& operator<< (ostream& os, const Vector<T>& v) {
  typename Vector<T>::const_iterator j;
  for (j=v.begin() ; j!= v.end(); j++){
    os << " " << std::setw(3) << static_cast<const T&>(*j);
  }
  os << std::endl;
  return os;
};

template <typename T>
inline std::istream& operator >> (std::istream &is, Vector<T> &v){
  typename Vector<T>::iterator j;
  for (j=v.begin() ; j!= v.end(); j++){
    is >> static_cast< T&>(*j) ;
  }
  return is;
}
#endif
